@@ -1,6 +1,6 @@
 package YATT::Lite::Constants;
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use 5.010; no if $] >= 5.017011, warnings => "experimental";
 
 require Carp;
