@@ -1,12 +1,13 @@
 package YATT::Lite::Factory;
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use Carp;
 use YATT::Lite::Breakpoint;
 sub MY () {__PACKAGE__}
 
 use 5.010;
 use Scalar::Util qw(weaken);
+use Encode qw/decode/;
 
 use parent qw/YATT::Lite::NSBuilder File::Spec/;
 use File::Path ();
@@ -31,14 +32,22 @@ use YATT::Lite::MFields
  , [cf_index_name =>
     (doc => "Rootname of index template. (default: index)")]
 
+ , [cf_ext_public =>
+    (doc => "public file extension for yatt. (default: yatt)")]
+
+ , [cf_ext_private =>
+    (doc => "hidden file extension for yatt. (default: ytmpl)")]
+
  , [cf_header_charset =>
     (doc => "Charset for outgoing HTTP Content-Type. (default: utf-8)")]
 
  , [cf_tmpl_encoding =>
-    (doc => "Perl encoding used while reading yatt templates. (default: '')")]
+    (doc => "Perl encoding used while reading yatt templates. (default: 'utf-8')")]
 
  , [cf_output_encoding =>
-    (doc => "Perl encoding used for outgoing response body. (default: '')")]
+    (doc => "Perl encoding used for outgoing response body."
+     ." Also this is used to decode incoming request parameters and PATH_INFO."
+     ." (default: 'utf-8')")]
 
  , [cf_offline =>
     (doc => "Whether header should be emitted or not.")]
@@ -46,6 +55,15 @@ use YATT::Lite::MFields
  , [cf_binary_config   =>
     (doc => "(This may be changed in future release) Whether .htyattconfig.* should be read with encoding or not.")]
 
+ , [cf_no_unicode =>
+    (doc => "(Compatibility option) Avoid use of utf8.")]
+
+ , [cf_no_unicode_params =>
+    (doc => "(Compatibility option) Avoid encoding conversion of input params.")]
+
+ , [cf_use_subpath =>
+    (doc => "pass sub-path_info")]
+
  , qw/
        cf_allow_missing_dir
 
@@ -53,6 +71,9 @@ use YATT::Lite::MFields
        loc2yatt
        path2yatt
 
+       loc2psgi_re
+       loc2psgi_dict
+
        tmpl_cache
 
        cf_debug_cgen
@@ -241,15 +262,32 @@ sub after_new {
   (my MY $self) = @_;
   $self->SUPER::after_new;
   $self->{cf_index_name} //= $self->default_index_name;
-  $self->{cf_output_encoding} //= $self->default_output_encoding;
-  $self->{cf_header_charset} //= (
-    $self->{cf_output_encoding} || $self->default_header_charset
-  );
+  $self->{cf_ext_public} //= $self->default_ext_public;
+  $self->{cf_ext_private} //= $self->default_ext_private;
+  if ($self->{cf_no_unicode}) {
+    $self->{cf_no_unicode_params} = 1;
+    $self->{cf_binary_config} = 1;
+    $self->{cf_header_charset}
+      //= ($self->{cf_output_encoding} || $self->default_header_charset);
+    $self->{cf_output_encoding}
+      //= $self->compat_default_output_encoding;
+  } else {
+    $self->{cf_header_charset}
+      //= ($self->{cf_output_encoding} // $self->default_header_charset);
+    $self->{cf_tmpl_encoding}
+      //= ($self->{cf_output_encoding} // $self->default_tmpl_encoding);
+    $self->{cf_output_encoding} //= $self->default_output_encoding;
+  }
+  $self->{cf_use_subpath} //= 1;
 }
 
-sub default_output_encoding { '' }
+sub compat_default_output_encoding { '' }
+sub default_output_encoding { 'utf-8' }
 sub default_header_charset  { 'utf-8' }
+sub default_tmpl_encoding   { 'utf-8' }
 sub default_index_name { 'index' }
+sub default_ext_public {'yatt'}
+sub default_ext_private {'ytmpl'}
 
 sub _after_after_new {
   (my MY $self) = @_;
@@ -277,14 +315,20 @@ sub _after_after_new {
 #========================================
 
 sub render {
+  my MY $self = shift;
+  my $raw_bytes = $self->render_encoded(@_);
+  decode(utf8 => $raw_bytes);
+}
+
+sub render_encoded {
   (my MY $self, my ($reqrec, $args, @opts)) = @_;
   # [$path_info, $subpage, $action]
   my ($path_info, @rest) = ref $reqrec ? @$reqrec : $reqrec;
 
-  my ($tmpldir, $loc, $file, $trailer)
-    = my @pi = lookup_path($path_info
-			   , $self->{tmpldirs}
-			   , $self->{cf_index_name}, ".yatt");
+  $path_info =~ s,^/*,/,;
+
+  my ($tmpldir, $loc, $file, $trailer, $is_index)
+    = my @pi = $self->lookup_split_path_info($path_info);
   unless (@pi) {
     die "No such location: $path_info";
   }
@@ -295,7 +339,7 @@ sub render {
 
   my $con = $self->make_simple_connection
   (
-    \@pi, yatt => $dh, noheader => 1
+    \@pi, yatt => $dh, noheader => 1, path_info => $path_info
     , $self->make_debug_params($reqrec, $args)
   );
 
@@ -310,6 +354,89 @@ sub render {
   $con->buffer;
 }
 
+sub lookup_split_path_info {
+  (my MY $self, my $path_info) = @_;
+  lookup_path($path_info
+	      , $self->{tmpldirs}
+	      , $self->{cf_index_name}, ".$self->{cf_ext_public}"
+	      , $self->{cf_use_subpath});
+}
+
+#========================================
+
+sub K_MOUNT_MATCH () { "__yatt" }
+
+sub lookup_psgi_mount {
+  (my MY $self, my $path_info) = @_;
+  $self->{loc2psgi_re} // $self->rebuild_psgi_mount;
+  $path_info =~ $self->{loc2psgi_re}
+    or return;
+  my @mount_match = grep {/^@{[K_MOUNT_MATCH()]}/o} keys %+
+    or return;
+  if (@mount_match >= 2) {
+    croak "Multiple match found for psgi_mount: \n"
+      . join("\n  ", map {$self->{loc2psgi_dict}{$_}[0]} @mount_match);
+  }
+
+  my $path_prefix = $+{$mount_match[0]};
+
+  my $item = $self->{loc2psgi_dict}{$path_prefix};
+
+  wantarray ? @{$item}[1..$#$item] : $item->[2];
+}
+
+sub mount_psgi {
+  (my MY $self, my ($path_prefix, $app, @opts)) = @_;
+  unless (defined $path_prefix) {
+    croak "path_prefix is empty! mount_psgi(path_prefix, psgi_app)";
+  }
+  if (not ref $path_prefix) {
+    $path_prefix =~ s,^/*,/,;
+  }
+  my $dict = $self->{loc2psgi_dict} //= +{};
+  my $key = K_MOUNT_MATCH() . (keys %$dict);
+  $dict->{$path_prefix} = [$key => $path_prefix => $app, @opts];
+
+  undef $self->{loc2psgi_re};
+
+  # For cascading call
+  $self;
+}
+
+sub rebuild_psgi_mount {
+  (my MY $self) = @_;
+  my @re;
+  foreach my $path_prefix (keys %{$self->{loc2psgi_dict} //= +{}}) {
+    my ($key, undef, $app) = @{$self->{loc2psgi_dict}{$path_prefix}};
+    push @re, qr{(?<$key>$path_prefix)};
+  }
+  my $all = join("|", @re);
+  $self->{loc2psgi_re} = qr{^(?:$all)(?:/|$)};
+}
+
+#========================================
+
+sub mount_action {
+  (my MY $self, my ($path_info, $action)) = @_;
+  if (my $ref = ref $path_info) {
+    croak "mount_action doesn't support $ref path_info, sorry";
+  }
+  my ($tmpldir, $loc, $file, $trailer, $is_index)
+    = my @pi = $self->lookup_split_path_info($path_info);
+  unless (@pi) {
+    croak "Can't find acutal directory for $path_info";
+  }
+  unless ($is_index) {
+    croak "Conflicting mount_action($path_info) with file=$file\n";
+  }
+  my $realdir = $tmpldir.$loc;
+  my $dh = $self->get_dirhandler($realdir);
+  $dh->set_action_handler($trailer, $action);
+
+  # For cascading call.
+  $self;
+}
+
 #========================================
 
 sub Connection () {'YATT::Lite::Connection'};
@@ -543,7 +670,9 @@ sub buildns {
 }
 
 sub _cf_delegates {
-  qw(output_encoding
+  qw(no_unicode
+     no_unicode_params
+     output_encoding
      header_charset
      tmpl_encoding
      debug_cgen
@@ -551,7 +680,11 @@ sub _cf_delegates {
      app_root
      namespace
      index_name
-     only_parse);
+     ext_public
+     ext_private
+     only_parse
+     use_subpath
+  );
 }
 
 sub configparams_for {
