@@ -1,6 +1,6 @@
 package YATT::Lite::WebMVC0::Connection; sub PROP () {__PACKAGE__}
 use strict;
-use warnings FATAL => qw(all);
+use warnings qw(FATAL all NONFATAL misc);
 use Carp;
 
 use base qw(YATT::Lite::Connection);
@@ -13,6 +13,8 @@ use YATT::Lite::MFields
 
     cf_no_nested_query
 
+    cf_no_unicode_params
+
     current_user
    /);
 use YATT::Lite::Util qw(globref url_encode nonempty lexpand);
@@ -25,20 +27,20 @@ use YATT::Lite::Util::CGICompat;
 BEGIN {
   # print STDERR join("\n", sort(keys our %FIELDS)), "\n";
 
-  foreach my $name (qw(raw_body)) {
+  foreach my $name (qw(raw_body uploads upload)) {
     *{globref(PROP, $name)} = sub {
       my PROP $prop = (my $glob = shift)->prop;
       unless ($prop->{cf_is_psgi}) {
 	croak "Connection method $name is PSGI mode only!"
       }
-      $prop->{cf_cgi}->$name;
+      $prop->{cf_cgi}->$name(@_);
     };
   }
 
   foreach my $name (qw(url_param)) {
     *{globref(PROP, $name)} = sub {
       my PROP $prop = (my $glob = shift)->prop;
-      $prop->{cf_cgi}->$name;
+      $prop->{cf_cgi}->$name(@_);
     };
   }
 
@@ -158,14 +160,23 @@ sub is_form_content_type {
   my ($self, $real_ct) = @_;
   return 1 if ($real_ct // '') eq '';
   foreach my $check_ct ($self->form_content_types) {
-    return 1 if $check_ct eq $real_ct;
+    return 1 if $real_ct =~ $check_ct;
   }
   return 0;
 }
 
 sub form_content_types {
-  qw(multipart/form-data
-     application/x-www-form-urlencoded);
+  (qr(^multipart/form-data\s*(?:;|$))i
+   , qr(^application/x-www-form-urlencoded$)i);
+}
+
+sub parse_nested_query {
+  my PROP $prop = (my $glob = shift)->prop;
+  my ($obj_or_string) = @_;
+  YATT::Lite::Util::parse_nested_query
+    ($obj_or_string
+     , (!$prop->{cf_no_unicode_params} && $prop->{cf_encoding})
+   );
 }
 
 sub convert_array_param_psgi {
@@ -174,8 +185,8 @@ sub convert_array_param_psgi {
   my Env $env = $prop->{cf_env};
   $prop->{params_hash} = do {
     if ($env->{CONTENT_TYPE} and defined $env->{CONTENT_LENGTH}) {
-      my $body = YATT::Lite::Util::parse_nested_query($req->raw_body);
-      my $qs = YATT::Lite::Util::parse_nested_query($env->{QUERY_STRING});
+      my $body = $glob->parse_nested_query([$req->body_parameters->flatten]);
+      my $qs = $glob->parse_nested_query($env->{QUERY_STRING});
       foreach my $key (keys %$qs) {
 	if (exists $body->{$key}) {
 	  die $glob->error("Attempt to overwrite post param '%s' by qs"
@@ -185,7 +196,7 @@ sub convert_array_param_psgi {
       }
       $body;
     } else {
-      YATT::Lite::Util::parse_nested_query($env->{QUERY_STRING});
+      $glob->parse_nested_query($env->{QUERY_STRING});
     }
   };
 }
@@ -195,7 +206,7 @@ sub convert_array_param_cgi {
   my ($cgi) = @_;
   return if ($cgi->content_type // "") eq "application/json";
   $prop->{params_hash}
-    = YATT::Lite::Util::parse_nested_query($cgi->query_string);
+    = $glob->parse_nested_query($cgi->query_string);
 }
 
 # Location(path part of url) of overall SiteApp.
